"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _jschardet = _interopRequireDefault(require("jschardet"));

var _iconvLite = _interopRequireDefault(require("iconv-lite"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

// skip iconv warning
_iconvLite.default.skipDecodeWarning = true;
/**
 * @desc CSV
 * @export
 * @class CSV
 */

var CSV =
/**
 * @desc schema
 * @type {Array<Schema>}
 * @memberof CSV
 */

/**
 * @desc encoding
 * @type {string}
 * @memberof CSV
 */

/**
 * @desc File Blob
 * @type {?Blob}
 * @memberof CSV
 */

/**
 * @desc csv data string
 * @type {?string}
 * @memberof CSV
 */

/**
 * @desc generate csv with header
 * @type {boolean}
 * @memberof CSV
 */

/**
 * Creates an instance of CSV.
 * @param {CSVOption} option option option
 * @param {Array<Schema>} option.schema schema
 * @param {string} [option.encoding=utf8] encoding
 * @param {boolean} [option.widthHeader=true] with header
 * @memberof CSV
 */
function CSV(_option) {
  var _this = this;

  _classCallCheck(this, CSV);

  _defineProperty(_defineProperty(_defineProperty(_defineProperty(_defineProperty(_defineProperty(_defineProperty(_defineProperty(_defineProperty(_defineProperty(_defineProperty(this, "checkSchema", function (schema) {
    var validSchema = schema.every(function (item) {
      return item.key;
    });

    if (!validSchema) {
      throw new TypeError('Invalid schema presented');
    }
  }), "checkEncoding", function (encoding) {
    var validEncoding = _iconvLite.default.encodingExists(encoding);

    if (!validEncoding) {
      throw new TypeError("Encoding Error: ".concat(encoding, " is not supported"));
    }
  }), "findSchemaByKey", function (key) {
    return _this.schema.find(function (item) {
      return item.key === key;
    }) || {
      key: key,
      type: 'string',
      label: key
    };
  }), "format", function (value, key) {
    var schema = _this.findSchemaByKey(key);

    switch (schema.type) {
      case 'number':
        return parseFloat(value);

      case 'date':
        return new Date(value).toString();

      case 'boolean':
        return (!!value).toString();

      case 'custom':
      case 'string':
      default:
        return value;
    }
  }), "encode", function (str) {
    var encoding = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : _this.encoding;

    _this.checkEncoding(encoding);

    return _iconvLite.default.encode(str, encoding);
  }), "decode", function (buf) {
    var encoding = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : _this.encoding;

    _this.checkEncoding(encoding);

    return _iconvLite.default.decode(buf, encoding);
  }), "convert", function (items) {
    var option = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    var encoding = option.encoding;
    var columns = _this.schema;
    var csvArray = [];
    var header = [];
    var keys = [];
    columns.forEach(function (column) {
      keys.push(column.key);
      header.push('"' + (column.label || column.key) + '"');
    });

    if (_this.withHeader) {
      csvArray.push(header);
    }

    items.forEach(function (item) {
      csvArray.push(keys.map(function (key) {
        return '"' + _this.format(item[key], key) + '"';
      }).join(','));
    });
    var str = csvArray.join('\n');

    var res = _this.encode(str, encoding);

    _this.data = _this.decode(res, encoding);
    _this.blob = new Blob([res], {
      type: 'text/csv'
    });
    return _this.data;
  }), "parse", function (buf) {
    var option = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

    if (Array.isArray(buf)) {
      _this.data = _this.convert(buf, option);
    } else {
      try {
        var res = _jschardet.default.detect(buf);

        var encoding = res.encoding;
        _this.data = _this.decode(buf, encoding);
      } catch (error) {
        throw new Error('Parse failed, please check input data');
      }
    }

    return _this.data;
  }), "getDataURL", function () {
    /* istanbul ignore next */
    if (!_this.blob) {
      throw new Error('No data');
    } else {
      return URL.createObjectURL(_this.blob);
    }
  }), "toJSON", function () {
    var columns = _this.schema;

    if (!_this.data) {
      return [];
    } else {
      var data = _this.data.replace(/\\r\\n|\\r/g, '\n').replace(/;|\\t|\|\^/g, ',');

      var temp = data.split('\n'); // TODO: header check

      temp.shift(); // remove empty line

      temp = temp.filter(function (temp) {
        return temp;
      });
      temp = temp.map(function (str) {
        var item = {};
        var arr = str.split(',');
        arr.map(function (s, index) {
          s = s.substring(1, s.length - 1);
          var column = columns[index];

          switch (column.type) {
            case 'number':
              item[column.key] = parseFloat(s);
              break;

            case 'boolean':
              var val;

              if (s === 'true') {
                val = true;
              } else if (s === 'false') {
                val = false;
              } else {
                val = !!s;
              }

              item[column.key] = val;
              break;

            case 'date':
              item[column.key] = new Date(s);
              break;

            case 'string':
            case 'custom':
            default:
              item[column.key] = s;
          }
        });
        return item;
      });
      return temp;
    }
  }), "toString", function () {
    return _this.data;
  });

  var _schema = _option.schema,
      _option$encoding = _option.encoding,
      _encoding = _option$encoding === void 0 ? 'utf8' : _option$encoding,
      _option$withHeader = _option.withHeader,
      withHeader = _option$withHeader === void 0 ? true : _option$withHeader;

  this.checkSchema(_schema);
  this.checkEncoding(_encoding);
  this.schema = _schema;
  this.encoding = _encoding;
  this.blob = null;
  this.data = '';
  this.withHeader = !!withHeader;
}
/**
 * @desc check schema
 * @throws {TypeError} Invalid schema presented
 * @memberof CSV
 */
;

exports.default = CSV;
module.exports = exports["default"];